////////////////////////////////////////////////////////////
// MainDialog.h - 
//
// Copyright (C) Alax.Info, 2006-2007
// http://alax.info
//
// A permission to use the source code is granted as long as reference to 
// source website http://alax.info is retained.

#pragma once

#include <atlcrack.h>
#include <atlmisc.h>
#include "aiwtlcrack.h"
#include "aicontrols.h"

////////////////////////////////////////////////////////////
// CMainDialog

class CMainDialog : 
	public CDialogImpl<CMainDialog>
{
public:

	enum { IDD = IDD_MAIN };

BEGIN_MSG_MAP_EX(CMainDialog)
	MSG_WM_INITDIALOG(OnInitDialog)
	COMMAND_ID_HANDLER_EX(IDOK, OnCommand)
	COMMAND_ID_HANDLER_EX(IDCANCEL, OnCommand)
	COMMAND_HANDLER_EX(IDC_LISTBOX, LBN_DBLCLK, OnListBoxDblClk)
	COMMAND_HANDLER_EX(IDC_COMBOBOX, CBN_SELENDOK, OnComboBoxSelEndOk)
	MSG_LVN_GETDISPINFO(IDC_LISTVIEW, OnListGetDispInfo)
	MSG_LVN_DBLCLK(IDC_LISTVIEW, OnListViewDblClk)
	MSG_TVN_GETDISPINFO(IDC_TREEVIEW, OnTreeGetDispInfo)
	MSG_TVN_DBLCLK(IDC_TREEVIEW, OnTreeViewDblClk)
	REFLECT_NOTIFICATIONS()
END_MSG_MAP()

public:

	////////////////////////////////////////////////////////
	// CListBoxItemData

	class CListBoxItemData
	{
	public:
		CString m_sText;
	};

	////////////////////////////////////////////////////////
	// CComboBoxItemData

	class CComboBoxItemData
	{
	public:
		CString m_sText;
	};

	////////////////////////////////////////////////////////
	// CListViewItemData

	class CListViewItemData
	{
	public:
		CString m_sText;

	public:
	// CListViewItemData
		CListViewItemData() throw()
		{
		}
		CListViewItemData(SIZE_T nIndex)
		{
			m_sText.Format(_T("List View Item %d"), nIndex);
		}
	};

	////////////////////////////////////////////////////////
	// CTreeViewItemData

	class CTreeViewItemData
	{
	public:
		CString m_sText;

	public:
	// CTreeViewItemData
		CTreeViewItemData() throw()
		{
		}
		CTreeViewItemData(SIZE_T nIndex)
		{
			m_sText.Format(_T("Tree View Item %d"), nIndex);
		}
	};

private:
	CAtlArray<CListBoxItemData> m_ListBoxItemDataArray;
	CAtlArray<CComboBoxItemData> m_ComboBoxItemDataArray;
	CAiListBoxT<CListBoxItemData*> m_ListBox;
	CAiComboBoxT<CComboBoxItemData*> m_ComboBox;
	CAiListViewT<CListViewItemData, CAiListControlDataTraitsT> m_ListView;
	CAiTreeViewT<CTreeViewItemData, CAiListControlDataTraitsT> m_TreeView;

public:
// CMainDialog

// Window message handlers
	LRESULT OnInitDialog(HWND, LPARAM)
	{
		// Initialize m_ListBoxItemDataArray
		{
			for(SIZE_T nIndex = 0; nIndex < 100; nIndex++)
			{
				CListBoxItemData& ListBoxItemData = m_ListBoxItemDataArray.GetAt(m_ListBoxItemDataArray.Add());
				ListBoxItemData.m_sText.Format(_T("List Box Item %d"), nIndex + 1);
			}
		}
		// Initialize m_ComboBoxItemDataArray
		{
			for(SIZE_T nIndex = 0; nIndex < 100; nIndex++)
			{
				CComboBoxItemData& ComboBoxItemData = m_ComboBoxItemDataArray.GetAt(m_ComboBoxItemDataArray.Add());
				ComboBoxItemData.m_sText.Format(_T("Combo Box Item %d"), nIndex + 1);
			}
		}
		// Initialize m_ListBox
		{
			m_ListBox = GetDlgItem(IDC_LISTBOX);
			for(SIZE_T nIndex = 0; nIndex < m_ListBoxItemDataArray.GetCount(); nIndex++)
				m_ListBox.AddString(m_ListBoxItemDataArray[nIndex].m_sText, &m_ListBoxItemDataArray[nIndex]);
			m_ListBox.SetCurSel(0);
		}
		// Initialize m_ComboBox
		{
			m_ComboBox = GetDlgItem(IDC_COMBOBOX);
			for(SIZE_T nIndex = 0; nIndex < m_ComboBoxItemDataArray.GetCount(); nIndex++)
				m_ComboBox.AddString(m_ComboBoxItemDataArray[nIndex].m_sText, &m_ComboBoxItemDataArray[nIndex]);
			m_ComboBox.SetCurSel(0);
		}
		// Initialize m_ListView
		{
			m_ListView = GetDlgItem(IDC_LISTVIEW);
			m_ListView.InsertColumn(0, _T("Text"), LVCFMT_LEFT, 240, -1);
			for(SIZE_T nIndex = 0; nIndex < 100; nIndex++)
				m_ListView.InsertItem((INT) nIndex, CListViewItemData(nIndex));
		}
		// Initialize m_TreeView
		{
			m_TreeView = GetDlgItem(IDC_TREEVIEW);
			static const SIZE_T g_pnParentIndexes[] = { (SIZE_T) -1, 0, 0, 0, 1, 1, 1, 4, 4, 4 };
			HTREEITEM phItems[_countof(g_pnParentIndexes)];
			phItems[0] = m_TreeView.InsertItem(NULL, NULL, CTreeViewItemData(0));
			for(SIZE_T nIndex = 1; nIndex < _countof(g_pnParentIndexes); nIndex++)
				phItems[nIndex] = m_TreeView.InsertItem(phItems[g_pnParentIndexes[nIndex]], NULL, CTreeViewItemData(nIndex));
			m_TreeView.Expand(phItems[0], TVE_EXPAND);
		}
		SetIcon(AtlLoadIconImage(IDR_MAINFRAME, GetSystemMetrics(SM_CXICON), GetSystemMetrics(SM_CYICON), LR_DEFAULTCOLOR), TRUE);
		SetIcon(AtlLoadIconImage(IDR_MAINFRAME, GetSystemMetrics(SM_CXSMICON), GetSystemMetrics(SM_CYSMICON), LR_DEFAULTCOLOR), FALSE);
		ATLVERIFY(CenterWindow());
		return TRUE;
	}
	LRESULT OnCommand(UINT, INT nIdentifier, HWND)
	{
		ATLVERIFY(EndDialog(nIdentifier));
		return 0;
	}
	LRESULT OnListBoxDblClk(UINT, INT, HWND)
	{
		INT nItem = m_ListBox.GetCurSel();
		if(nItem < 0)
			return 0;
		CListBoxItemData* pListBoxItemData = m_ListBox.GetItemData(nItem);
		AtlMessageBox(m_hWnd, (LPCTSTR) pListBoxItemData->m_sText, _T("List Box Selected Item Data"), MB_ICONINFORMATION | MB_OK);
		return 0;
	}
	LRESULT OnComboBoxSelEndOk(UINT, INT, HWND)
	{
		INT nItem = m_ComboBox.GetCurSel();
		if(nItem < 0)
			return 0;
		CComboBoxItemData* pComboBoxItemData = m_ComboBox.GetItemData(nItem);
		AtlMessageBox(m_hWnd, (LPCTSTR) pComboBoxItemData->m_sText, _T("Combo Box Selected Item Data"), MB_ICONINFORMATION | MB_OK);
		return 0;
	}
	LRESULT OnListGetDispInfo(NMLVDISPINFO* pHeader)
	{
		if(pHeader->item.mask & LVIF_TEXT)
		{
			m_ListView.m_sTextBuffer.Empty();
			CListViewItemData& ListViewItemData = m_ListView.DataFromParameter(pHeader->item.lParam);
			m_ListView.m_sTextBuffer = ListViewItemData.m_sText;
			pHeader->item.pszText = m_ListView.GetTextBuffer();
		}
		return 0;
	}
	LRESULT OnListViewDblClk(NMITEMACTIVATE*)
	{
		INT nItem = m_ListView.GetNextItem(-1, LVNI_SELECTED);
		if(nItem >= 0)
			AtlMessageBox(m_hWnd, (LPCTSTR) m_ListView.GetItemData(nItem).m_sText, _T("List View First Selected Item Data"), MB_ICONINFORMATION | MB_OK);
		return 0;
	}
	LRESULT OnTreeGetDispInfo(NMTVDISPINFO* pHeader)
	{
		if(pHeader->item.mask & TVIF_TEXT)
		{
			m_TreeView.m_sTextBuffer.Empty();
			CTreeViewItemData& TreeViewItemData = m_TreeView.DataFromParameter(pHeader->item.lParam);
			m_TreeView.m_sTextBuffer = TreeViewItemData.m_sText;
			pHeader->item.pszText = m_TreeView.GetTextBuffer();
		}
		return 0;
	}
	LRESULT OnTreeViewDblClk(NMHDR*)
	{
		HTREEITEM hItem = m_TreeView.GetSelectedItem();
		if(hItem)
			AtlMessageBox(m_hWnd, (LPCTSTR) m_TreeView.GetItemData(hItem).m_sText, _T("Tree View First Selected Item Data"), MB_ICONINFORMATION | MB_OK);
		return 0;
	}
};
