////////////////////////////////////////////////////////////
// aicontrols.h -
//
// Copyright (C) Alax.Info, 2006-2007
// http://alax.info
//
// A permission to use the source code is granted as long as reference to 
// source website http://alax.info is retained.

#pragma once

#include <atlcoll.h>

////////////////////////////////////////////////////////////
// CAiSimpleControlDataTraitsT

template <typename _Data, typename _DataTraits = CElementTraits<_Data> >
class CAiSimpleControlDataTraitsT
{
public:
	typedef typename _DataTraits _DataTraitsClass;
	typedef typename _DataTraits::INARGTYPE DATAINARGTYPE;
	//typedef typename _DataTraits::OUTARGTYPE DATAOUTARGTYPE;
	typedef typename _Data DATAOUTARGTYPE;
	typedef typename _Data ITEMDATA;

protected:
	static DATAOUTARGTYPE DataFromItemData(ITEMDATA ItemData) throw()
	{
		return ItemData;
	}
	static ITEMDATA ConstructData(SIZE_T nIndex) throw()
	{
		nIndex;
		return (ITEMDATA) NULL;
	}
	static ITEMDATA ConstructData(SIZE_T nIndex, DATAINARGTYPE Data) throw()
	{
		nIndex;
		return Data;
	}
	static VOID DestructData(ITEMDATA ItemData)
	{
		ItemData;
	}
	static VOID DestructAllData() throw()
	{
	}
	ITEMDATA UpdateData(ITEMDATA ItemData, DATAINARGTYPE Data)
	{
		ItemData;
		return Data;
	}

public:
// CAiSimpleControlDataTraitsT
};

////////////////////////////////////////////////////////////
// CAiListControlDataTraitsT

template <typename _Data, typename _DataTraits = CElementTraits<_Data> >
class CAiListControlDataTraitsT
{
public:
	typedef typename _DataTraits _DataTraitsClass;
	typedef typename _DataTraits::INARGTYPE DATAINARGTYPE;
	typedef typename _DataTraits::OUTARGTYPE DATAOUTARGTYPE;
	typedef POSITION ITEMDATA;

private:
	CAtlList<_Data> m_DataList;

protected:
	VOID AssertItemDataValid(ITEMDATA ItemData) const throw()
	{
#if defined(_DEBUG)
		for(POSITION Position = m_DataList.GetHeadPosition(); Position; m_DataList.GetNext(Position))
			if(Position == ItemData)
				return;
		ATLASSERT(FALSE);
#endif // defined(_DEBUG)
	}
	DATAOUTARGTYPE DataFromItemData(ITEMDATA ItemData) throw()
	{
		ATLASSERT(ItemData);
		AssertItemDataValid(ItemData);
		return m_DataList.GetAt(ItemData);
	}
	ITEMDATA ConstructData(SIZE_T nIndex)
	{
		if(nIndex < m_DataList.GetCount())
			return m_DataList.InsertBefore(m_DataList.FindIndex(nIndex), typename _Data());
		return m_DataList.AddTail();
	}
	ITEMDATA ConstructData(SIZE_T nIndex, DATAINARGTYPE Data)
	{
		if(nIndex < m_DataList.GetCount())
			return m_DataList.InsertBefore(m_DataList.FindIndex(nIndex), Data);
		return m_DataList.AddTail(Data);
	}
	VOID DestructData(ITEMDATA ItemData)
	{
		m_DataList.RemoveAt(ItemData);
	}
	VOID DestructAllData() throw()
	{
		m_DataList.RemoveAll();
	}
	ITEMDATA UpdateData(ITEMDATA ItemData, DATAINARGTYPE Data)
	{
		if(!ItemData)
			return ConstructData(Data);
		AssertItemDataValid(ItemData);
		m_DataList.GetAt(ItemData) = Data;
		return ItemData;
	}

public:
// CAiSimpleControlDataTraitsT
	CAtlList<_Data>& GetDataList() throw()
	{
		return m_DataList;
	}
	DATAOUTARGTYPE DataFromParameter(LPARAM lParam) throw()
	{
		return DataFromItemData((ITEMDATA) lParam);
	}
};

////////////////////////////////////////////////////////////
// CAiListBoxT

template <typename _Data, template <typename _Data, typename _DataTraits> class _ControlDataTraitsT = CAiSimpleControlDataTraitsT, typename _DataTraits = CElementTraits<_Data>, typename _Base = CListBox>
class CAiListBoxT :
	public _Base,
	public _ControlDataTraitsT<_Data, _DataTraits>
{
public:
	typedef typename _ControlDataTraitsT<_Data, _DataTraits> _ControlDataTraits;
	typedef typename _ControlDataTraitsT<_Data, _DataTraits>::_DataTraitsClass _DataTraitsClass;
	typedef typename _ControlDataTraitsT<_Data, _DataTraits>::DATAINARGTYPE DATAINARGTYPE;
	typedef typename _ControlDataTraitsT<_Data, _DataTraits>::DATAOUTARGTYPE DATAOUTARGTYPE;
	typedef typename _ControlDataTraitsT<_Data, _DataTraits>::ITEMDATA ITEMDATA;

private:
	VOID* GetItemDataPtr(INT nIndex) const;
	INT SetItemDataPtr(INT nIndex, VOID* pvItemData);

public:
// CAiListBoxT
	CAiListBoxT& operator = (HWND hWindow) throw()
	{
		_Base::operator = (hWindow);
		return *this;
	}
	VOID Initialize()
	{
		ATLASSERT(IsWindow());
		CAtlArray<CString> StringArray;
		_StringHelper::GetLines(_StringHelper::Load(GetDlgCtrlID()), StringArray);
		ResetContent();
		for(SIZE_T nIndex = 0; nIndex < StringArray.GetCount(); nIndex++)
			__super::SetItemData(AddString(StringArray[nIndex]), (DWORD_PTR) nIndex);
	}
	VOID Initialize(HWND hWindow)
	{
		ATLASSERT(!IsWindow());
		m_hWnd = hWindow;
		Initialize();
	}
	ITEMDATA GetItemParameter(INT nItem) const throw()
	{
		return (ITEMDATA) __super::GetItemData(nItem);
	}
	VOID SetItemParameter(INT nItem, ITEMDATA ItemData) throw()
	{
		ATLVERIFY(__super::SetItemData(nItem, (DWORD_PTR) ItemData) != CB_ERR);
	}
	INT FindItemData(DATAINARGTYPE Data) //const
	{
		INT nCount = GetCount();
		for(INT nItem = 0; nItem < nCount; nItem++)
			if(_DataTraitsClass::CompareElements(GetItemData(nItem), Data))
				return nItem;
		return -1;
	}
	INT AddString(LPCTSTR pszString, DATAINARGTYPE Data)
	{
		INT nItem;
		ITEMDATA ItemData = ConstructData((SIZE_T) -1, Data);
		nItem = __super::AddString(pszString);
		ATLASSERT(nItem >= 0);
		if(nItem >= 0)
			__super::SetItemData(nItem, (DWORD_PTR) ItemData);
		else
			DestructData(ItemData);
		return nItem;
	}
	INT InsertString(INT nIndex, LPCTSTR pszString, DATAINARGTYPE Data)
	{
		INT nItem;
		ITEMDATA ItemData = ConstructData((SIZE_T) nIndex, Data);
		nItem = __super::InsertString(nIndex, pszString);
		ATLASSERT(nItem >= 0);
		if(nItem >= 0)
			__super::SetItemData(nItem, (DWORD_PTR) ItemData);
		else
			DestructData(ItemData);
		return nItem;
	}
	template <typename _Parameter>
	BOOL FindFirstItemThatT(BOOL (*pCriteria)(INT, DATAOUTARGTYPE, _Parameter), _Parameter Parameter, INT* pnItem = NULL, INT nStartItem = 0)
	{
		ATLASSERT(pCriteria);
		INT nItemCount = GetCount();
		for(INT nItem = 0; nItem < nItemCount; nItem++)
			if(pCriteria(nItem, GetItemData(nItem), Parameter))
			{
				if(pnItem)
					*pnItem = nItem;
				return TRUE;
			}
		return FALSE;
	}
	template <typename _Parameter>
	BOOL FindLastItemThatT(BOOL (*pCriteria)(INT, DATAOUTARGTYPE, _Parameter), _Parameter Parameter, INT* pnItem = NULL, INT nStartItem = -1)
	{
		ATLASSERT(pCriteria);
		for(INT nItem = (nStartItem < 0) ? GetCount() - 1 : nStartItem; nItem >= 0; nItem--)
			if(pCriteria(nItem, GetItemData(nItem), Parameter))
			{
				if(pnItem)
					*pnItem = nItem;
				return TRUE;
			}
		return FALSE;
	}
	template <typename _Parameter>
	INT GetItemCountThatT(BOOL (*pCriteria)(INT, DATAOUTARGTYPE, _Parameter), _Parameter Parameter)
	{
		ATLASSERT(pCriteria);
		INT nItemCount = GetCount();
		INT nCount = 0;
		for(INT nItem = 0; nItem < nItemCount; nItem++)
			if(pCriteria(nItem, GetItemData(nItem), Parameter))
				nCount++;
		return nCount;
	}

// CListBox
	DATAOUTARGTYPE GetItemData(INT nItem) throw()
	{
		return DataFromItemData((ITEMDATA) __super::GetItemData(nItem));
	}
	VOID SetItemData(INT nItem, DATAINARGTYPE Data)
	{
		ATLASSERT(nItem < GetCount());
		ITEMDATA CurrentItemData = (ITEMDATA) __super::GetItemData(nItem);
		ITEMDATA ItemData = UpdateData(CurrentItemData, Data);
		ATLVERIFY(__super::SetItemData(nItem, (DWORD_PTR) ItemData) != CB_ERR);
	}
	VOID ResetContent()
	{
		__super::ResetContent();
		DestructAllData();
	}
	INT AddString(LPCTSTR pszString)
	{
		INT nItem;
		ITEMDATA ItemData = ConstructData((SIZE_T) -1);
		nItem = __super::AddString(pszString);
		ATLASSERT(nItem >= 0);
		if(nItem >= 0)
			__super::SetItemData(nItem, (DWORD_PTR) ItemData);
		else
			DestructData(ItemData);
		return nItem;
	}
	BOOL DeleteString(INT nIndex) throw()
	{
		ITEMDATA ItemData = (ITEMDATA) __super::GetItemData(nItem);
		if(__super::DeleteString(nIndex) < 0)
			return FALSE;
		DestructData(ItemData);
		return TRUE;
	}
	INT InsertString(INT nIndex, LPCTSTR pszString)
	{
		INT nItem;
		ITEMDATA ItemData = ConstructData((SIZE_T) nIndex);
		nItem = __super::InsertString(nIndex, pszString);
		ATLASSERT(nItem >= 0);
		if(nItem >= 0)
			__super::SetItemData(nItem, (DWORD_PTR) ItemData);
		else
			DestructData(ItemData);
		return nItem;
	}
};

////////////////////////////////////////////////////////////
// CAiComboBoxT

template <typename _Data, template <typename _Data, typename _DataTraits> class _ControlDataTraitsT = CAiSimpleControlDataTraitsT, typename _DataTraits = CElementTraits<_Data>, typename _Base = CComboBox>
class CAiComboBoxT :
	public _Base,
	public _ControlDataTraitsT<_Data, _DataTraits>
{
public:
	typedef typename _ControlDataTraitsT<_Data, _DataTraits> _ControlDataTraits;
	typedef typename _ControlDataTraitsT<_Data, _DataTraits>::_DataTraitsClass _DataTraitsClass;
	typedef typename _ControlDataTraitsT<_Data, _DataTraits>::DATAINARGTYPE DATAINARGTYPE;
	typedef typename _ControlDataTraitsT<_Data, _DataTraits>::DATAOUTARGTYPE DATAOUTARGTYPE;
	typedef typename _ControlDataTraitsT<_Data, _DataTraits>::ITEMDATA ITEMDATA;

private:
	VOID* GetItemDataPtr(INT nIndex) const;
	INT SetItemDataPtr(INT nIndex, VOID* pvItemData);

public:
// CAiComboBoxT
	CAiComboBoxT& operator = (HWND hWindow) throw()
	{
		_Base::operator = (hWindow);
		return *this;
	}
	VOID Initialize()
	{
		ATLASSERT(IsWindow());
		CAtlArray<CString> StringArray;
		_StringHelper::GetLines(_StringHelper::Load(GetDlgCtrlID()), StringArray);
		ResetContent();
		for(SIZE_T nIndex = 0; nIndex < StringArray.GetCount(); nIndex++)
			__super::SetItemData(AddString(StringArray[nIndex]), (DWORD_PTR) nIndex);
	}
	VOID Initialize(HWND hWindow)
	{
		ATLASSERT(!IsWindow());
		m_hWnd = hWindow;
		Initialize();
	}
	ITEMDATA GetItemParameter(INT nItem) const throw()
	{
		return (ITEMDATA) __super::GetItemData(nItem);
	}
	VOID SetItemParameter(INT nItem, ITEMDATA ItemData) throw()
	{
		ATLVERIFY(__super::SetItemData(nItem, (DWORD_PTR) ItemData) != CB_ERR);
	}
	INT FindItemData(DATAINARGTYPE Data) //const
	{
		INT nCount = GetCount();
		for(INT nItem = 0; nItem < nCount; nItem++)
			if(_DataTraitsClass::CompareElements(GetItemData(nItem), Data))
				return nItem;
		return -1;
	}
	INT AddString(LPCTSTR pszString, DATAINARGTYPE Data)
	{
		INT nItem;
		ITEMDATA ItemData = ConstructData((SIZE_T) -1, Data);
		nItem = __super::AddString(pszString);
		ATLASSERT(nItem >= 0);
		if(nItem >= 0)
			__super::SetItemData(nItem, (DWORD_PTR) ItemData);
		else
			DestructData(ItemData);
		return nItem;
	}
	INT InsertString(INT nIndex, LPCTSTR pszString, DATAINARGTYPE Data)
	{
		INT nItem;
		ITEMDATA ItemData = ConstructData((SIZE_T) nIndex, Data);
		nItem = __super::InsertString(nIndex, pszString);
		ATLASSERT(nItem >= 0);
		if(nItem >= 0)
			__super::SetItemData(nItem, (DWORD_PTR) ItemData);
		else
			DestructData(ItemData);
		return nItem;
	}
	template <typename _Parameter>
	BOOL FindFirstItemThatT(BOOL (*pCriteria)(INT, DATAOUTARGTYPE, _Parameter), _Parameter Parameter, INT* pnItem = NULL, INT nStartItem = 0)
	{
		ATLASSERT(pCriteria);
		INT nItemCount = GetCount();
		for(INT nItem = 0; nItem < nItemCount; nItem++)
			if(pCriteria(nItem, GetItemData(nItem), Parameter))
			{
				if(pnItem)
					*pnItem = nItem;
				return TRUE;
			}
		return FALSE;
	}
	template <typename _Parameter>
	BOOL FindLastItemThatT(BOOL (*pCriteria)(INT, DATAOUTARGTYPE, _Parameter), _Parameter Parameter, INT* pnItem = NULL, INT nStartItem = -1)
	{
		ATLASSERT(pCriteria);
		for(INT nItem = (nStartItem < 0) ? GetCount() - 1 : nStartItem; nItem >= 0; nItem--)
			if(pCriteria(nItem, GetItemData(nItem), Parameter))
			{
				if(pnItem)
					*pnItem = nItem;
				return TRUE;
			}
		return FALSE;
	}
	template <typename _Parameter>
	INT GetItemCountThatT(BOOL (*pCriteria)(INT, DATAOUTARGTYPE, _Parameter), _Parameter Parameter)
	{
		ATLASSERT(pCriteria);
		INT nItemCount = GetCount();
		INT nCount = 0;
		for(INT nItem = 0; nItem < nItemCount; nItem++)
			if(pCriteria(nItem, GetItemData(nItem), Parameter))
				nCount++;
		return nCount;
	}

// CComboBox
	DATAOUTARGTYPE GetItemData(INT nItem) throw()
	{
		return DataFromItemData((ITEMDATA) __super::GetItemData(nItem));
	}
	VOID SetItemData(INT nItem, DATAINARGTYPE Data)
	{
		ATLASSERT(nItem < GetCount());
		ITEMDATA CurrentItemData = (ITEMDATA) __super::GetItemData(nItem);
		ITEMDATA ItemData = UpdateData(CurrentItemData, Data);
		ATLVERIFY(__super::SetItemData(nItem, (DWORD_PTR) ItemData) != CB_ERR);
	}
	VOID ResetContent()
	{
		__super::ResetContent();
		DestructAllData();
	}
	INT AddString(LPCTSTR pszString)
	{
		INT nItem;
		ITEMDATA ItemData = ConstructData((SIZE_T) -1);
		nItem = __super::AddString(pszString);
		ATLASSERT(nItem >= 0);
		if(nItem >= 0)
			__super::SetItemData(nItem, (DWORD_PTR) ItemData);
		else
			DestructData(ItemData);
		return nItem;
	}
	BOOL DeleteString(INT nIndex) throw()
	{
		ITEMDATA ItemData = (ITEMDATA) __super::GetItemData(nItem);
		if(__super::DeleteString(nIndex) < 0)
			return FALSE;
		DestructData(ItemData);
		return TRUE;
	}
	INT InsertString(INT nIndex, LPCTSTR pszString)
	{
		INT nItem;
		ITEMDATA ItemData = ConstructData((SIZE_T) nIndex);
		nItem = __super::InsertString(nIndex, pszString);
		ATLASSERT(nItem >= 0);
		if(nItem >= 0)
			__super::SetItemData(nItem, (DWORD_PTR) ItemData);
		else
			DestructData(ItemData);
		return nItem;
	}
};

////////////////////////////////////////////////////////////
// CAiListViewT

template <typename _Data, template <typename _Data, typename _DataTraits> class _ControlDataTraitsT = CAiSimpleControlDataTraitsT, typename _DataTraits = CElementTraits<_Data>, typename _Base = CListViewCtrl>
class CAiListViewT :
	public _Base,
	public _ControlDataTraitsT<_Data, _DataTraits>
{
public:
	typedef typename _ControlDataTraitsT<_Data, _DataTraits> _ControlDataTraits;
	typedef typename _ControlDataTraitsT<_Data, _DataTraits>::_DataTraitsClass _DataTraitsClass;
	typedef typename _ControlDataTraitsT<_Data, _DataTraits>::DATAINARGTYPE DATAINARGTYPE;
	typedef typename _ControlDataTraitsT<_Data, _DataTraits>::DATAOUTARGTYPE DATAOUTARGTYPE;
	typedef typename _ControlDataTraitsT<_Data, _DataTraits>::ITEMDATA ITEMDATA;

public:
	CString m_sTextBuffer;

public:
// CAiListViewT
	CAiListViewT& operator = (HWND hWindow) throw()
	{
		_Base::operator = (hWindow);
		return *this;
	}
	ITEMDATA GetItemParameter(INT nItem) const throw()
	{
		return (ITEMDATA) __super::GetItemData(nItem);
	}
	VOID SetItemParameter(INT nItem, ITEMDATA ItemData) throw()
	{
		ATLVERIFY(__super::SetItemData(nItem, (DWORD_PTR) ItemData));
	}
	INT FindItemByParameter(ITEMDATA ItemData, INT nStartItem = -1) const
	{
		LVFINDINFO FindInfo = { LVFI_PARAM };
		FindInfo.lParam = (LPARAM) ItemData;
		return __super::FindItem(&FindInfo, nStartItem);
	}
	INT FindItemData(DATAINARGTYPE Data) const
	{
		// WARN: In most cases LVM_FINDITEM with LVFI_PARAM would work faster, however 
		//       straightforward comparison allow to customize compare using traits class
		INT nCount = GetItemCount();
		for(INT nItem = 0; nItem < nCount; nItem++)
			if(_DataTraitsClass::CompareElements(GetItemData(nItem), Data))
				return nItem;
		return -1;
	}
	INT InsertItem(INT nItem, DATAINARGTYPE Data, LPCTSTR pszItem = LPSTR_TEXTCALLBACK, INT nImage = I_IMAGECALLBACK)
	{
		INT nInsertedItem;
		ITEMDATA ItemData = ConstructData((SIZE_T) nItem, Data);
		nInsertedItem = __super::InsertItem(nItem, pszItem, nImage);
		ATLASSERT(nInsertedItem >= 0);
		if(nInsertedItem >= 0)
			__super::SetItemData(nInsertedItem, (DWORD_PTR) ItemData);
		else
			DestructData(ItemData);
		return nInsertedItem;
	}
	CString& GetTextBufferString(BOOL bEmpty = FALSE) throw()
	{
		if(bEmpty)
			m_sTextBuffer.Empty();
		return m_sTextBuffer;
	}
	LPTSTR GetTextBuffer() const throw()
	{
		return const_cast<LPTSTR>((LPCTSTR) m_sTextBuffer);
	}
	template <typename _Parameter>
	BOOL FindFirstItemThatT(BOOL (*pCriteria)(INT, DATAOUTARGTYPE, _Parameter), _Parameter Parameter, INT* pnItem = NULL, INT nStartItem = 0)
	{
		ATLASSERT(pCriteria);
		INT nItemCount = GetItemCount();
		for(INT nItem = 0; nItem < nItemCount; nItem++)
			if(pCriteria(nItem, GetItemData(nItem), Parameter))
			{
				if(pnItem)
					*pnItem = nItem;
				return TRUE;
			}
		return FALSE;
	}
	template <typename _Parameter>
	BOOL FindLastItemThatT(BOOL (*pCriteria)(INT, DATAOUTARGTYPE, _Parameter), _Parameter Parameter, INT* pnItem = NULL, INT nStartItem = -1)
	{
		ATLASSERT(pCriteria);
		for(INT nItem = (nStartItem < 0) ? GetItemCount() - 1 : nStartItem; nItem >= 0; nItem--)
			if(pCriteria(nItem, GetItemData(nItem), Parameter))
			{
				if(pnItem)
					*pnItem = nItem;
				return TRUE;
			}
		return FALSE;
	}
	template <typename _Parameter>
	INT GetItemCountThatT(BOOL (*pCriteria)(INT, DATAOUTARGTYPE, _Parameter), _Parameter Parameter)
	{
		ATLASSERT(pCriteria);
		INT nItemCount = GetItemCount();
		INT nCount = 0;
		for(INT nItem = 0; nItem < nItemCount; nItem++)
			if(pCriteria(nItem, GetItemData(nItem), Parameter))
				nCount++;
		return nCount;
	}

// CListViewCtrl
	DATAOUTARGTYPE GetItemData(INT nItem) throw()
	{
		return DataFromItemData((ITEMDATA) __super::GetItemData(nItem));
	}
	VOID SetItemData(INT nItem, DATAINARGTYPE Data)
	{
		ATLASSERT(nItem < GetCount());
		ITEMDATA CurrentItemData = (ITEMDATA) __super::GetItemData(nItem);
		ITEMDATA ItemData = UpdateData(CurrentItemData, Data);
		ATLVERIFY(__super::SetItemData(nItem, (DWORD_PTR) ItemData));
	}
	INT InsertItem(const LVITEM* pItem)
	{
		INT nInsertedItem;
		ITEMDATA ItemData = ConstructData((SIZE_T) pItem->iItem);
		nInsertedItem = __super::InsertItem(pItem);
		ATLASSERT(nInsertedItem >= 0);
		if(nInsertedItem < 0)
			__super::SetItemData(nInsertedItem, (DWORD_PTR) ItemData);
		else
			DestructData(ItemData);
		return nInsertedItem;
	}
	INT InsertItem(UINT nMask, INT nItem, LPCTSTR pszItem, UINT nState, UINT nStateMask, INT nImage, LPARAM lParam)
	{
		LVITEM Item;
		ZeroMemory(&Item, sizeof Item);
		Item.mask = nMask;
		Item.iItem = nItem;
		Item.state = nState;
		Item.stateMask = nStateMask;
		Item.pszText = pszItem;
		Item.iImage = nImage;
		Item.lParam = lParam;
		return InsertItem(&Item);
	}
	INT InsertItem(INT nItem, LPCTSTR pszItem)
	{
		return InsertItem(LVIF_TEXT, nItem, pszItem, 0, 0, 0, 0);
	}
	INT InsertItem(INT nItem, LPCTSTR pszItem, INT nImage)
	{
		return InsertItem(LVIF_TEXT, nItem, pszItem, 0, 0, nImage, 0);
	}
	BOOL DeleteItem(INT nItem)
	{
		ITEMDATA ItemData = (ITEMDATA) __super::GetItemData(nItem);
		if(!__super::DeleteItem(nItem))
			return FALSE;
		DestructData(ItemData);
		return TRUE;
	}
	BOOL DeleteAllItems()
	{
		if(!__super::DeleteAllItems())
			return FALSE;
		DestructAllData();
		return TRUE;
	}
	INT AddItem(INT nItem, INT nSubItem, LPCTSTR pszItem, INT nImageIndex = -1)
	{
		LVITEM Item;
		ZeroMemory(&Item, sizeof Item);
		Item.mask = LVIF_TEXT | ((nImageIndex >= 0) ? LVIF_IMAGE : 0);
		Item.iItem = nItem;
		Item.iSubItem = nSubItem;
		Item.pszText = pszItem;
		Item.iImage = nImageIndex;
		if(nSubItem)
			return SetItem(&Item) ? nItem : -1;
		return InsertItem(&Item);
	}
};

////////////////////////////////////////////////////////////
// CAiTreeViewT

template <typename _Data, template <typename _Data, typename _DataTraits> class _ControlDataTraitsT = CAiSimpleControlDataTraitsT, typename _DataTraits = CElementTraits<_Data>, typename _Base = CTreeViewCtrl>
class CAiTreeViewT :
	public _Base,
	public _ControlDataTraitsT<_Data, _DataTraits>
{
public:
	typedef typename _ControlDataTraitsT<_Data, _DataTraits> _ControlDataTraits;
	typedef typename _ControlDataTraitsT<_Data, _DataTraits>::_DataTraitsClass _DataTraitsClass;
	typedef typename _ControlDataTraitsT<_Data, _DataTraits>::DATAINARGTYPE DATAINARGTYPE;
	typedef typename _ControlDataTraitsT<_Data, _DataTraits>::DATAOUTARGTYPE DATAOUTARGTYPE;
	typedef typename _ControlDataTraitsT<_Data, _DataTraits>::ITEMDATA ITEMDATA;

public:
	CString m_sTextBuffer;

public:
// CAiTreeViewT
	CAiTreeViewT& operator = (HWND hWindow) throw()
	{
		_Base::operator = (hWindow);
		return *this;
	}
	ITEMDATA GetItemParameter(HTREEITEM hItem) const throw()
	{
		return (ITEMDATA) __super::GetItemData(hItem);
	}
	VOID SetItemParameter(HTREEITEM hItem, ITEMDATA ItemData) throw()
	{
		ATLVERIFY(__super::SetItemData(hItem, (DWORD_PTR) ItemData));
	}
	//INT FindItemByParameter(ITEMDATA ItemData, INT nStartItem = -1) const
	//{
	//	LVFINDINFO FindInfo = { LVFI_PARAM };
	//	FindInfo.lParam = (LPARAM) ItemData;
	//	return __super::FindItem(&FindInfo, nStartItem);
	//}
	//INT FindItemData(DATAINARGTYPE Data) const
	//{
	//	// WARN: In most cases LVM_FINDITEM with LVFI_PARAM would work faster, however 
	//	//       straightforward comparison allow to customize compare using traits class
	//	INT nCount = GetItemCount();
	//	for(INT nItem = 0; nItem < nCount; nItem++)
	//		if(_DataTraitsClass::CompareElements(GetItemData(nItem), Data))
	//			return nItem;
	//	return -1;
	//}
	HTREEITEM InsertItem(HTREEITEM hParentItem, HTREEITEM hInsertAfterItem, DATAINARGTYPE Data, LPCTSTR pszItem = LPSTR_TEXTCALLBACK, INT nImage = I_IMAGECALLBACK, INT nSelectedImage = I_IMAGECALLBACK)
	{
		HTREEITEM hInsertedItem;
		ITEMDATA ItemData = ConstructData((SIZE_T) -1, Data);
		TVINSERTSTRUCT InsertStruct;
		ZeroMemory(&InsertStruct, sizeof InsertStruct);
		InsertStruct.hParent = hParentItem;
		InsertStruct.hInsertAfter = hInsertAfterItem;
		InsertStruct.item.mask = TVIF_TEXT | ((nImage >= 0) ? TVIF_IMAGE : 0) | TVIF_PARAM | ((nSelectedImage >= 0) ? TVIF_SELECTEDIMAGE : 0);
		InsertStruct.item.pszText = const_cast<LPTSTR>(pszItem);
		InsertStruct.item.iImage = nImage;
		InsertStruct.item.lParam = (DWORD_PTR) ItemData;
		InsertStruct.item.iSelectedImage = nSelectedImage;
		hInsertedItem = __super::InsertItem(&InsertStruct);
		ATLASSERT(hInsertedItem);
		if(!hInsertedItem)
			DestructData(ItemData);
		return hInsertedItem;
	}
	CString& GetTextBufferString(BOOL bEmpty = FALSE) throw()
	{
		if(bEmpty)
			m_sTextBuffer.Empty();
		return m_sTextBuffer;
	}
	LPTSTR GetTextBuffer() const throw()
	{
		return const_cast<LPTSTR>((LPCTSTR) m_sTextBuffer);
	}
	//template <typename _Parameter>
	//BOOL FindFirstItemThatT(BOOL (*pCriteria)(INT, DATAOUTARGTYPE, _Parameter), _Parameter Parameter, INT* pnItem = NULL, INT nStartItem = 0)
	//{
	//	ATLASSERT(pCriteria);
	//	INT nItemCount = GetItemCount();
	//	for(INT nItem = 0; nItem < nItemCount; nItem++)
	//		if(pCriteria(nItem, GetItemData(nItem), Parameter))
	//		{
	//			if(pnItem)
	//				*pnItem = nItem;
	//			return TRUE;
	//		}
	//	return FALSE;
	//}
	//template <typename _Parameter>
	//BOOL FindLastItemThatT(BOOL (*pCriteria)(INT, DATAOUTARGTYPE, _Parameter), _Parameter Parameter, INT* pnItem = NULL, INT nStartItem = -1)
	//{
	//	ATLASSERT(pCriteria);
	//	for(INT nItem = (nStartItem < 0) ? GetItemCount() - 1 : nStartItem; nItem >= 0; nItem--)
	//		if(pCriteria(nItem, GetItemData(nItem), Parameter))
	//		{
	//			if(pnItem)
	//				*pnItem = nItem;
	//			return TRUE;
	//		}
	//	return FALSE;
	//}
	//template <typename _Parameter>
	//INT GetItemCountThatT(BOOL (*pCriteria)(INT, DATAOUTARGTYPE, _Parameter), _Parameter Parameter)
	//{
	//	ATLASSERT(pCriteria);
	//	INT nItemCount = GetItemCount();
	//	INT nCount = 0;
	//	for(INT nItem = 0; nItem < nItemCount; nItem++)
	//		if(pCriteria(nItem, GetItemData(nItem), Parameter))
	//			nCount++;
	//	return nCount;
	//}

// CTreeViewCtrl
	DATAOUTARGTYPE GetItemData(HTREEITEM hItem) throw()
	{
		return DataFromItemData((ITEMDATA) __super::GetItemData(hItem));
	}
	VOID SetItemData(HTREEITEM hItem, DATAINARGTYPE Data)
	{
		ATLASSERT(hItem);
		ITEMDATA CurrentItemData = (ITEMDATA) __super::GetItemData(hItem);
		ITEMDATA ItemData = UpdateData(CurrentItemData, Data);
		ATLVERIFY(__super::SetItemData(hItem, (DWORD_PTR) ItemData));
	}
	HTREEITEM InsertItem(const TVINSERTSTRUCT* pInsertStruct)
	{
		HTREEITEM hInsertedItem;
		ITEMDATA ItemData = ConstructData((SIZE_T) -1);
		hInsertedItem = __super::InsertItem(const_cast<TVINSERTSTRUCT*>(pInsertStruct));
		ATLASSERT(hInsertedItem);
		if(hInsertedItem)
			__super::SetItemData(hInsertedItem, (DWORD_PTR) ItemData);
		else
			DestructData(ItemData);
		return hInsertedItem;
	}
	HTREEITEM InsertItem(UINT nMask, LPCTSTR pszItem, INT nImage, INT nSelectedImage, UINT nState, UINT nStateMask, LPARAM lParam, HTREEITEM hParentItem, HTREEITEM hInsertAfterItem)
	{
		ATLASSERT(!(nMask & TVIF_PARAM) && !lParam);
		TVINSERTSTRUCT InsertStruct;
		ZeroMemory(&InsertStruct, sizeof InsertStruct);
		InsertStruct.hParent = hParentItem;
		InsertStruct.hInsertAfter = hInsertAfterItem;
		InsertStruct.item.mask = nMask;
		InsertStruct.item.pszText = const_cast<LPTSTR>(pszItem);
		InsertStruct.item.iImage = nImage;
		InsertStruct.item.iSelectedImage = nSelectedImage;
		InsertStruct.item.state = nState;
		InsertStruct.item.stateMask = nStateMask;
		//InsertStruct.item.lParam = lParam;
		return InsertItem(&InsertStruct);
	}
	HTREEITEM InsertItem(LPCTSTR pszItem, INT nImage, INT nSelectedImage, HTREEITEM hParentItem, HTREEITEM hInsertAfterItem)
	{
		return InsertItem(TVIF_TEXT | TVIF_IMAGE | TVIF_SELECTEDIMAGE, pszItem, nImage, nSelectedImage, 0, 0, 0, hParentItem, hInsertAfterItem); 
	}
	HTREEITEM InsertItem(LPCTSTR pszItem, HTREEITEM hParentItem, HTREEITEM hInsertAfterItem)
	{
		return InsertItem(TVIF_TEXT, pszItem, 0, 0, 0, 0, 0, hParentItem, hInsertAfterItem);
	}
	BOOL DeleteItem(HTREEITEM hItem)
	{
		ITEMDATA ItemData = (ITEMDATA) __super::GetItemData(nItem);
		if(!__super::DeleteItem(hItem))
			return FALSE;
		DestructData(ItemData);
		return TRUE;
	}
	BOOL DeleteAllItems()
	{
		if(!__super::DeleteAllItems())
			return FALSE;
		DestructAllData();
		return TRUE;
	}
};

